/*
 * PowerSaver CPX
 * Written by Tommy Andersen 8-9 February 1998
 */

/*
 * It will compile with Lattice 5.52 and with a TURBO/PURE C
 * compatible aesbind.h
 * Allways compile with 16 bit integers
 */


#include <stdio.h>
#include <stdlib.h>
#include <strings.h>
#include <aesbind.h>
#include <vdi.h>
#include <cpx.h>
#include "rpow.h"


/* some defines */

#define TRUE  1
#define FALSE 0
#define FL3DBAK 0x0400
#define FL3DACT 0x0600
/* Speciel Lattice stuff */
#define AES_VERSION _AESglobal[0]
#define GLOBAL _AESglobal
#define INTIN  _AESintin
#define INTOUT _AESintout
#define ADDRIN _AESaddrin

/* prototypes */

CPXINFO *	__stdargs __saveds cpx_init( XCPB *Xcpb);
int  			__stdargs __saveds cpx_call( GRECT *rect);
void 			set_powersaver(void);
void 			save_options(void);
int 			is_powersave_installed(void);
void			popup_onoff(GRECT *);
unsigned long read_timer_input(void);
static int	has_3d(void);
static int	appl_xgetinfo(int type,int *out1,int *out2,int *out3,int *out4);
static int	appl_getinfo(int ap_gtype,int *ap_gout1,int *ap_gout2,int *ap_gout3,int *ap_gout4);


/* This struture is the same as in powersav.prg (TSR)
 */
struct TSR_power
	{	unsigned short	version;
		unsigned long	ticks;
		unsigned short	active;
		unsigned long	active_time;	/* ticks */
	};

struct Save_Config
	{	unsigned long	ticks;
		unsigned short active;
	};

/* warning, special Lattice feature!
 * This structure must be the very first 'far' variable
 * or the save function will not work
 * Preset, 5min to blank, and activated
 */	
struct Save_Config far cpx_config = {60000U,TRUE};
struct TSR_power *PowerSaver = NULL;

XCPB    *xcpb;
CPXINFO cpxinfo = { cpx_call, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };

int magx = 0;				/* Magx not found yet      */
int aes_version = 0;		/* default unknown version */
int colors = 2;			/* expect only two colors  */


/*
 *
 *  At last... here we start
 *
 */

CPXINFO * __stdargs __saveds cpx_init(XCPB *Xcpb)
{
	xcpb = Xcpb;
	
	if (xcpb->booting)
	{
		if ( is_powersave_installed() )
		{
			PowerSaver->active = cpx_config.active;
			PowerSaver->ticks = cpx_config.ticks;
		}
		return ( (CPXINFO *) 1 );
	}
	if (!xcpb->SkipRshFix)
		xcpb->SkipRshFix=1;			/* resource set is fixed by the dercs program */
	return &cpxinfo;
}


int  __stdargs __saveds cpx_call(GRECT *rect)
{
	int msg[8];
	register int finished = FALSE;
	register int button = 0;
	register int redraw = FALSE;
	register OBJECT *tree = POWERSAVBOX;
	
	/* Set coordinates */
	tree[POWSAV_ROOT].ob_x = rect->g_x;
	tree[POWSAV_ROOT].ob_y = rect->g_y;

	/* Initialise the PowerSaver pointer */
	is_powersave_installed();

	if (PowerSaver)
	{	sprintf(tree[POWSAV_MINUTS].ob_spec.tedinfo->te_ptext,"%3d",\
			(unsigned short) ( (unsigned long)(PowerSaver->ticks)/12000U) );
		strcpy(tree[POWSAV_ONOFF].ob_spec.free_string,(PowerSaver->active)?"on":"off" );
		sprintf(tree[POWSAV_VERSION].ob_spec.tedinfo->te_ptext,\
			"Version: %x.%02x",(PowerSaver->version)>>8,(PowerSaver->version & 0xff));
		sprintf(tree[POWSAV_ACTIVE].ob_spec.tedinfo->te_ptext,"Saved %ld min.",\
			(unsigned long)(PowerSaver->active_time/12000U) );
	}
	else
	{	strcpy(tree[POWSAV_MINUTS].ob_spec.tedinfo->te_ptext,"N/A" );
		strcpy(tree[POWSAV_ONOFF].ob_spec.free_string,"off" );
		strcpy(tree[POWSAV_VERSION].ob_spec.tedinfo->te_ptext,"Version: not installed");
		strcpy(tree[POWSAV_ACTIVE].ob_spec.tedinfo->te_ptext,"Saved: info not available");
	}

	if (!aes_version && appl_init()>=0)
		aes_version = AES_VERSION;		/* do that only one time */

	/* now.. make some 3D and colors if available */

	if (has_3d())	/* vdi/aes is just initialised */
	{	tree[POWSAV_SAVE].ob_flags |= FL3DACT;
		tree[POWSAV_OK].ob_flags |= FL3DACT;
		tree[POWSAV_CANCEL].ob_flags |= FL3DACT;
		tree[POWSAV_ONOFF].ob_flags |= FL3DACT;	/* try 3D on popup too */

		tree[POWSAV_ROOT].ob_flags |= FL3DBAK;
		tree[POWSAV_OKBOX].ob_flags |= FL3DBAK;
		tree[POWSAV_SAVEBOX].ob_flags |= FL3DBAK;
		tree[POWSAV_TOPBOX].ob_flags |= FL3DBAK;

		if (magx)			
		{	tree[POWSAV_MINUTS].ob_state &= (~OUTLINED);
			tree[POWSAV_MINUTS].ob_flags |= FL3DBAK;
		}
		else
		{	tree[POWSAV_MINUTS].ob_state = (tree[POWSAV_MINUTS].ob_state|SELECTED) & ~OUTLINED;
			tree[POWSAV_MINUTS].ob_flags |= FL3DACT;
		}
	}
	if (colors>=15)
	{
		tree[POWSAV_LOGO].ob_spec.bitblk->bi_color = RED;
		tree[POWSAV_FREE].ob_spec.bitblk->bi_color = LBLUE;
		tree[POWSAV_AUTHOR].ob_spec.tedinfo->te_color = (BLUE<<8)|WHITE;
	}

	objc_draw(tree, POWSAV_ROOT, MAX_DEPTH, rect->g_x,rect->g_y,rect->g_w,rect->g_h );

	while (!finished)
	{
		button = (*xcpb->Xform_do)(tree, 0, msg);
		if (button != -1)					/* a button was selected */
			button &= 0x7FFF;				/* double clicks off */
		if (button == -1)					/* a messages to us */
		{
			switch (msg[0])
			{
				case AC_CLOSE:				/* ac_close = cancel */
					button = POWSAV_CANCEL;
					break;
				case WM_CLOSED:			/* wm_close = ok */
					button = POWSAV_OK;
					break;
			}
		}
		if (button != -1)
		{
			tree[button].ob_state &= ~SELECTED;
			switch (button)
			{
				case POWSAV_OK:
					set_powersaver();
					finished = TRUE;
					break;
				case POWSAV_CANCEL:
					finished = TRUE;
					break;
				case POWSAV_SAVE:
					save_options();
					redraw = TRUE;
					break;
				case POWSAV_ONOFF:
					popup_onoff(rect);
					redraw = TRUE;										
					break;
			}
			if (redraw)
			{	objc_draw(tree, button, 1, rect->g_x,rect->g_y,rect->g_w,rect->g_h);
				redraw = FALSE;
			}
		}
	}
	return 0;
}


/*
 *  Save the input from dialog to TSR program
 */
 
void set_powersaver(void)
{
	register unsigned long ticks;
	
	if (is_powersave_installed())
	{	ticks = read_timer_input();
		if (!strcmp("on",POWERSAVBOX[POWSAV_ONOFF].ob_spec.free_string))
			PowerSaver->active = TRUE;
		else
			PowerSaver->active = FALSE;
		if ( ticks == 0)
			PowerSaver->active = FALSE;
		PowerSaver->ticks = ticks;
	}
}


/*
 * The user pressed the Save button
 */

void save_options(void)
{
	register unsigned long ticks;

	/* read values from dialog and save */
	ticks = read_timer_input();
	if (!strcmp("on",POWERSAVBOX[POWSAV_ONOFF].ob_spec.free_string))	
		cpx_config.active = TRUE;
	else
		cpx_config.active = FALSE;
	if ( ticks == 0)
		cpx_config.active = FALSE;
	cpx_config.ticks = ticks;

	if ((*xcpb->XGen_Alert)(0))	/* ask before saving */
		if (!(*xcpb->CPX_Save)(&cpx_config,sizeof(cpx_config)))
			(*xcpb->XGen_Alert)(2);	/* Alert with error */
}


/*
 * Find the TSR program and initialise PowerSaver pointer
 */
 
int is_powersave_installed(void)
{
	if ( (*xcpb->getcookie)('TASC',(long*)&PowerSaver) )
		return TRUE;
	return FALSE;
}


/*
 * Call popup, user can choose on or off
 */

void popup_onoff(GRECT *rect)
{
	int a;
	char *pop[] =	{
							" on ",
							" off" 
						};
	GRECT clip;

	objc_xywh(POWERSAVBOX, POWSAV_ONOFF,&clip);

	a = (*xcpb->Popup)(pop,2,-1,IBM,&clip,rect);
	if (-1 < a)
		strcpy(POWERSAVBOX[POWSAV_ONOFF].ob_spec.free_string,(!a) ? "on":"off");
}


/*
 * return user selected inactivity timer (ticks) from dialog
 */

unsigned long read_timer_input(void)
{
	register unsigned int minuts = (unsigned int)atol(POWERSAVBOX[POWSAV_MINUTS].ob_spec.tedinfo->te_ptext);
	
	if (minuts > 999)
		minuts = 999;
	
	return (unsigned long)minuts * 12000U;
}


/*
 * Now, can we make the dialog 3D?
 */

static int has_3d(void)
{
	long value;
	int info1,info2,dummy;
	int handle;
	int work_in[11]= {1,1,1,1,1,1,1,1,1,1,2};
	int work_out[57];

	handle=graf_handle(&dummy,&dummy,&dummy,&dummy);
	v_opnvwk(work_in,&handle,work_out);
	if (handle!=0)
	{	colors=work_out[13];
		v_clsvwk(handle);
	}

	/* Speciel MagiC */
	if ((*xcpb->getcookie)('MagX',(long*)&value) && value!=0l)
	{
		magx = ((int **) value)[2][24];
		if (magx < 0x0200)
			magx = 0;
	}
	appl_xgetinfo(13,&info1,&info2,&dummy,&dummy);

	/* return 3d flag... Wonder why it's so complicated? */
	return (aes_version>=0x332 && 
	      (!magx || (magx>=0x0300 && colors>=16 && info1))) || (info1 && info2);
}

static int appl_xgetinfo(int type,int *out1,int *out2,int *out3,int *out4)
{
	static int has_agi = -1;
	int back = 0;

	if (has_agi<0)
		has_agi = (aes_version>=0x400 || magx || appl_find("?AGI\0\0\0\0")==0) ? 1 : 0;

	if (has_agi>0) /* AES is good */
		back = appl_getinfo(type,out1,out2,out3,out4);

	if (!back)
		*out1 = *out2 = *out3 = *out4 = 0;

	return (back);
}


/*
 * This aes call is not implemented in Lattice 5.52 gemlib
 * Someday I will rewrite that gemlib.
 */
extern short _AESglobal[];
short AESCALL(void**,short);
#pragma inline d0=AESCALL( d1,d0) { "4e42"; }

static int appl_getinfo(int ap_gtype,int *ap_gout1,int *ap_gout2,int *ap_gout3,int *ap_gout4)
{
	register int *array;

	INTIN[0] = ap_gtype;
	_AEScontrol[3]=0;
	_AEScontrol[2]=5;
	_AEScontrol[1]=1;
	_AEScontrol[0]=0x82;
	AESCALL( _AESpb,200);	/*_aes(0,0x82010500L); */

	array = &INTOUT[5];
	*ap_gout4 = *--array;
	*ap_gout3 = *--array;
	*ap_gout2 = *--array;
	*ap_gout1 = *--array;
	return(*--array);
}


/* that's it folks! */
